from typing import Union, Optional, List

from fastapi import FastAPI, Request
from fastapi.responses import RedirectResponse
from pydantic import BaseModel

from getcohorts.core import (
    get_seed as _get_seed,
    get_cohort as _get_cohort,
    DEFAULT_COHORTS
)
from getcohorts import __version__

docs_data = {
    'version': __version__,
    'title': "GetCohorts",
    'description': "Utilities for allocating users to cohorts of an A/B test."
}

app = FastAPI(redoc_url=None, docs_url=None)
v1 = FastAPI(redoc_url='/', **docs_data)


class SeedParams(BaseModel):
    identifier: Union[int, str, float]
    experiment: Union[int, str, float]


class Seed(BaseModel):
    params: SeedParams
    seed: int


class CohortParams(BaseModel):
    identifier: Union[int, str, float]
    experiment: Union[int, str, float]
    cohorts: Optional[List[str]] = DEFAULT_COHORTS


class Cohort(BaseModel):
    params: CohortParams
    cohort: str


@app.middleware('http')
async def add_version(request: Request, call_next):
    resp = await call_next(request)
    resp.headers['X-API-Version'] = __version__
    return resp


@v1.get("/health")
def get_health():
    return {"healthy": True}


@v1.get('/seeds', response_model=Seed)
def get_seed(data: SeedParams):
    seed = _get_seed(**data.dict())
    return Seed(seed=seed, params=data)


@v1.get('/cohorts', response_model=Cohort)
def get_cohort(data: CohortParams):
    cohort = _get_cohort(**data.dict())
    return Cohort(cohort=cohort, params=data)


@app.get('/')
def get_index():
    return RedirectResponse('/v1/')


app.mount('/v1', v1)



from random import Random
from typing import Any, List
import hashlib


DEFAULT_COHORTS = ['experimental', 'control']


def _get_bytes(value: Any) -> bytes:
    if type(value) == bytes:
        return value
    elif type(value) == str:
        return value.encode('utf8')
    elif type(value) == float:
        if value == int(value):
            return str(int(value)).encode('utf8')
        else:
            return str(value).encode('utf8')
    else:
        return str(value).encode('utf8')


def _format_cohorts(cohorts: List[str]):
    return list(sorted(set(cohorts)))


def get_seed(identifier: Any, experiment: Any) -> int:
    """
    Convert any combination of identifier and experiment to a random integer.

    Calling this function with the same identifier and experiment value
    will return the same integer every time.
    """

    identifier = _get_bytes(identifier)
    experiment = _get_bytes(experiment)
    hexdigest = hashlib.md5(identifier + experiment).hexdigest()
    seed = int(hexdigest, 16)
    return seed


def get_cohort(identifier: Any, experiment: Any, cohorts:
               List[str] = DEFAULT_COHORTS):
    """
    Randomly assign a value from the `cohort` list to the given experiment and
    identifier.

    Calling this function with the same identifier, experiment, and list of
    cohorts will return the same selection from the cohort list every time.
    """

    seed = get_seed(identifier, experiment)
    formatted_cohorts = _format_cohorts(cohorts)
    random = Random()
    random.seed(seed)
    index = int(random.random() * len(formatted_cohorts))
    return formatted_cohorts[index]



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="getcohorts",  # Replace with your own username
    version="0.1.17",
    author="Tom Waterman",
    author_email="tjwaterman99@gmail.com",
    description="Utilities for randomizing A/B tests",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    install_requires=[
        'gunicorn==20.0.4',
        'fastapi==0.61.1',
        'uvicorn==0.11.8',
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7',
    url="http://docs.getcohorts.com/"
)
